@@ -34,6 +34,7 @@
 #include <gmodule.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-transport.h>
+#include <libgnomeprint/gnome-print-path.h>
 #include <stdio.h>
 
 static void gnome_print_transport_class_init (GnomePrintTransportClass *klass);
@@ -268,7 +269,7 @@ gnome_print_transport_new_from_module_name (const gchar *module_name, GnomePrint
 	module = g_hash_table_lookup (modules, module_name);
 
 	if (!module) {
-		gchar *dir = g_build_filename (GNOME_PRINT_MODULES_DIR, "transports", NULL);
+		gchar *dir = g_build_filename (gnome_print_modules_dir, "transports", NULL);
 		insert = TRUE;
 		path = g_module_build_path (dir, module_name);
 		module = g_module_open (path, G_MODULE_BIND_LAZY);
@@ -278,7 +279,7 @@ gnome_print_transport_new_from_module_name (const gchar *module_name, GnomePrint
 	if (!module) {
 		insert = TRUE;
 		g_free (path);
-		path = g_module_build_path (GNOME_PRINT_MODULES_DIR, module_name);
+		path = g_module_build_path (gnome_print_modules_dir, module_name);
 		module = g_module_open (path, G_MODULE_BIND_LAZY);
 	}
 	
@@ -293,6 +294,8 @@ gnome_print_transport_new_from_module_name (const gchar *module_name, GnomePrint
 
 	if (!g_module_symbol (module, "gnome_print__transport_get_type", &get_type)) {
 		g_warning ("Missing gnome_print__transport_get_type in %s\n", path);
+		if (insert)
+			g_hash_table_remove (modules, module_name);
 		g_module_close (module);
 		if (path)
 			g_free (path);
@@ -302,6 +305,8 @@ gnome_print_transport_new_from_module_name (const gchar *module_name, GnomePrint
 	transport = gnome_print_transport_create (get_type, config);
 	if (transport == NULL) {
 		g_warning ("Could not create transport in %s\n", path);
+		if (insert)
+			g_hash_table_remove (modules, module_name);
 		g_module_close (module);
 		if (path)
 			g_free (path);
