@@ -29,6 +29,7 @@
 #include <string.h>
 #include <gmodule.h>
 #include <errno.h>
+#include <fcntl.h>
 
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-print-private.h>
@@ -37,15 +38,23 @@
 #include <libgnomeprint/gnome-print-ps2.h>
 #include <libgnomeprint/gnome-print-pdf.h>
 #include <libgnomeprint/gnome-print-frgba.h>
+#include <libgnomeprint/gnome-print-meta.h>
+#include <libgnomeprint/gnome-print-rbuf.h>
 
 #ifdef ENABLE_SVG
 #include <libgnomeprint/gnome-print-svg.h>
 #endif
 
+#ifdef HAVE_GDI
+#include <libgnomeprint/gnome-print-gdi.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <fcntl.h>
+gchar *gnome_print_locale_dir = GNOMELOCALEDIR;
+gchar *gnome_print_modules_dir = GNOME_PRINT_MODULES_DIR;
+gchar *gnome_print_data_dir = GNOME_PRINT_DATA_DIR;
 
 #ifdef HAVE_MMAP
 /* For the buffer stuff, remove when the buffer stuff is moved out here */
@@ -62,8 +71,16 @@
 #define HAVE_BROKEN_MMAP
 #endif
 
+#elif defined G_OS_WIN32
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <io.h>
 #endif /* HAVE_MMAP */
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 static void gnome_print_context_init (GnomePrintContext *pc);
 static void gnome_print_context_finalize (GObject *object);
@@ -138,7 +155,7 @@ gint
 gnome_print_context_construct (GnomePrintContext *pc, GnomePrintConfig *config)
 {
 	GnomePrintReturnCode retval = GNOME_PRINT_OK;
-	
+
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (config != NULL, GNOME_PRINT_ERROR_UNKNOWN);
@@ -149,7 +166,7 @@ gnome_print_context_construct (GnomePrintContext *pc, GnomePrintConfig *config)
 
 	if (GNOME_PRINT_CONTEXT_GET_CLASS (pc)->construct)
 		retval = GNOME_PRINT_CONTEXT_GET_CLASS (pc)->construct (pc);
-	
+
 	return retval;
 }
 
@@ -162,7 +179,7 @@ gnome_print_context_create_transport (GnomePrintContext *pc)
 	g_return_val_if_fail (pc->transport == NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
 	pc->transport = gnome_print_transport_new (pc->config);
-	
+
 	if (pc->transport == NULL) {
 		g_warning ("Could not create transport inside gnome_print_context_create_transport");
 		return GNOME_PRINT_ERROR_UNKNOWN;
@@ -193,7 +210,7 @@ int
 gnome_print_beginpage (GnomePrintContext *pc, const guchar *name)
 {
 	guchar *real_name;
-	
+
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
@@ -205,7 +222,7 @@ gnome_print_beginpage (GnomePrintContext *pc, const guchar *name)
 	} else {
 		real_name = (guchar *) name;
 	}
-	
+
 	gp_gc_reset (pc->gc);
 	pc->haspage = TRUE;
 
@@ -257,12 +274,12 @@ gnome_print_showpage (GnomePrintContext *pc)
  * gnome_print_end_doc:
  * @pc: A #GnomePrintContext
  *
- * to be called at the end of any copy of the document before the next 
+ * to be called at the end of any copy of the document before the next
  * copy starts. It will do such things as ejecting a page in duplex printing.
- * 
- * 
- * 
- * 
+ *
+ *
+ *
+ *
  *
  * Returns: #GNOME_PRINT_OK or positive value on success, negative error
  * code on failure.
@@ -273,7 +290,7 @@ gnome_print_end_doc (GnomePrintContext *pc)
 	gint ret;
 
 	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), 
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc),
 			      GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_BADCONTEXT);
 	g_return_val_if_fail (!pc->haspage, GNOME_PRINT_ERROR_NOMATCH);
@@ -471,6 +488,19 @@ gnome_print_context_close (GnomePrintContext *pc)
 	return ret;
 }
 
+GnomePrintContext *
+gnome_print_context_new_from_module_name (const gchar *module_name)
+{
+	if (!module_name) return NULL;
+
+	if (!strcmp (module_name, "rbuf"))
+		return g_object_new (GNOME_TYPE_PRINT_RBUF, NULL);
+	if (!strcmp (module_name, "meta"))
+		return g_object_new (GNOME_TYPE_PRINT_META, NULL);
+
+	return NULL;
+}
+
 /**
  * gnome_print_context_new:
  * @config: GnomePrintConfig object to query print settings from
@@ -511,7 +541,7 @@ gnome_print_context_new (GnomePrintConfig *config)
 		g_free (drivername);
 		return pc;
 	}
-	
+
 	if (strcmp (drivername, "gnome-print-pdf") == 0) {
 		pc = gnome_print_pdf_new (config);
 		if (pc == NULL)
@@ -530,6 +560,16 @@ gnome_print_context_new (GnomePrintConfig *config)
 	}
 #endif
 
+#ifdef HAVE_GDI
+	if (strcmp (drivername, "gnome-print-gdi") == 0) {
+		pc = gnome_print_gdi_new (config);
+		if (pc == NULL)
+			return NULL;
+		g_free (drivername);
+		return pc;
+	}
+#endif
+
 	if (pc == NULL)
 		g_warning ("Could not create context for driver: %s", drivername);
 
@@ -547,6 +587,10 @@ gnome_print_buffer_munmap (GnomePrintBuffer *b)
 		if (b->was_mmaped)
 			munmap (b->buf, b->buf_size);
 		else
+#elif defined G_OS_WIN32
+		if (b->was_mmaped)
+			UnmapViewOfFile (b->buf);
+		else
 #endif
 			g_free (b->buf);
 	}
@@ -574,7 +618,7 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 	b->was_mmaped = FALSE;
 	b->fd = -1;
 
-	fh = open (file_name, O_RDONLY);
+	fh = open (file_name, O_RDONLY | O_BINARY);
 	if (fh < 0) {
 		g_warning ("Can't open \"%s\"", file_name);
 		return GNOME_PRINT_ERROR_UNKNOWN;
@@ -584,9 +628,27 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 		close (fh);
 		return GNOME_PRINT_ERROR_UNKNOWN;
 	}
-	
+
 #ifdef HAVE_MMAP
 	b->buf = mmap (NULL, s.st_size, PROT_READ, MAP_SHARED, fh, 0);
+#elif defined G_OS_WIN32
+	{
+		HANDLE handle;
+
+		handle = CreateFileMapping ((HANDLE) _get_osfhandle (fh), NULL, PAGE_READONLY, 0,
+		s.st_size, NULL);
+		if (!handle) {
+			g_warning ("Can't CreateFileMapping file %s", file_name);
+			return GNOME_PRINT_ERROR_UNKNOWN;
+		}
+		b->buf = MapViewOfFile (handle, FILE_MAP_READ, 0, 0, 0);
+		CloseHandle (handle);
+		if (!b->buf) {
+			g_warning ("Can't MapViewOfFile file %s", file_name);
+			return GNOME_PRINT_ERROR_UNKNOWN;
+		}
+		b->buf_size = s.st_size;
+	}
 #endif
 
 	if ((b->buf == NULL) || (b->buf == (void *) -1)) {
@@ -595,10 +657,10 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 		b->buf = g_try_malloc (s.st_size);
 		b->buf_size = s.st_size;
 		if (b->buf != NULL) {
-			ssize_t nread, total_read;
-			
+			gssize nread, total_read;
+
 			nread = total_read = 0;
-			
+
 			while (total_read < s.st_size) {
 				nread = read (fh, b->buf + total_read,
 					      MIN (GP_MMAP_BUFSIZ, s.st_size - total_read));
@@ -612,7 +674,7 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 						b->buf_size = 0;
 						break; /* failure */
 				}
-				} else 
+				} else
 					total_read += nread;
 			}
 		}
@@ -635,3 +697,37 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 
 	return GNOME_PRINT_OK;
 }
+
+#ifdef G_OS_WIN32
+BOOL APIENTRY DllMain (HANDLE hModule, 
+                       DWORD  ul_reason_for_call, 
+                       LPVOID lpReserved)
+{
+	char *gnome_print_prefix;
+
+	switch (ul_reason_for_call)
+	{
+		case DLL_PROCESS_ATTACH:
+			gnome_print_prefix = g_win32_get_package_installation_directory (NULL, 
+				"libgnomeprint-2-2-0.dll");
+			if (gnome_print_prefix) {
+				gnome_print_locale_dir = g_build_filename (
+					gnome_print_prefix, "share", "locale", NULL);
+				gnome_print_modules_dir = g_build_filename (
+					gnome_print_prefix, "lib", "libgnomeprint", VERSION, "modules", NULL);
+				gnome_print_data_dir = g_build_filename (
+					gnome_print_prefix, "share", "libgnomeprint", VERSION, NULL);
+			}
+
+			break;
+		case DLL_PROCESS_DETACH:
+			g_free (gnome_print_locale_dir);
+			g_free (gnome_print_modules_dir);
+			g_free (gnome_print_data_dir);
+
+			break;
+	}
+
+	return TRUE;
+}
+#endif
