@@ -61,7 +61,9 @@ static struct poptOption options[] = {
 	  "Dump the GpaRoot node to the console",  NULL},
 	{ "debug",    '\0', POPT_ARG_NONE, &debug, 0,
 	  "Print debugging output",          NULL},
+#ifndef USE_POPT_DLL
 	POPT_AUTOHELP
+#endif
 	{ NULL }
 };
 
@@ -70,7 +72,7 @@ typedef enum {
 	TEST_GPA_ERROR = -2,
 	TEST_GPA_BAD_PARAMETERS = -1,
 	TEST_GPA_SUCCESS = 0,
-	TEST_GPA_SUCCESS_LAST = 99,
+	TEST_GPA_SUCCESS_LAST = 99
 } GpaTestRetval;
 
 typedef struct _GpaTestNum GpaTestNum;
@@ -698,7 +700,7 @@ parse_command_line (int argc, const char ** argv, gint *num)
 		GList *list = NULL;
 		/* We crash, this is part of the sanity check that allows us verify that crashes
 		 * are treated as errors when they happen (Chema)*/
-		g_print ("Crashing ...\n");
+		g_print ("Crashing (On purpose)...\n");
 		list->next = NULL;
 	}
 	
@@ -727,17 +729,21 @@ int
 main (int argc, const char * argv[])
 {
 	GpaTestRetval ret;
+#ifndef G_OS_WIN32
 	struct sigaction sig;
+#endif
 
 	g_type_init ();
 	loop = g_main_loop_new (NULL, FALSE);
 
 	parse_command_line (argc, argv, &num);
 
+#ifndef G_OS_WIN32
 	/* Catch sigsegv signals */
 	sig.sa_handler = handle_sigsegv;
 	sig.sa_flags = 0;
 	sigaction (SIGSEGV, &sig, NULL);
+#endif
 
 	gpa_init ();
 
